@@ -13,7 +13,6 @@ from threading import Thread, Event, current_thread
 import time
 import types
 import warnings
-from datetime import datetime
 from getpass import getpass
 from pprint import pprint
 
@@ -22,15 +21,14 @@ pjoin = os.path.join
 import zmq
 from zmq.eventloop.ioloop import IOLoop
 from zmq.eventloop.zmqstream import ZMQStream
-from tornado.gen import multi_future
 
 from traitlets.config.configurable import MultipleInstanceError
 from IPython.core.application import BaseIPythonApplication
 from IPython.core.profiledir import ProfileDir, ProfileDirError
 
+from IPython import get_ipython
 from IPython.utils.capture import RichOutput
 from IPython.utils.coloransi import TermColors
-from jupyter_client.jsonutil import extract_dates, parse_date
 from jupyter_client.localinterfaces import localhost, is_local_ip
 from IPython.paths import get_ipython_dir
 from IPython.utils.path import compress_user
@@ -47,9 +45,10 @@ from ipyparallel import util
 
 from jupyter_client.session import Session
 from ipyparallel import serialize
+from ipyparallel.serialize import PrePickled
 
 from .asyncresult import AsyncResult, AsyncHubResult
-from .futures import MessageFuture
+from .futures import MessageFuture, multi_future
 from .view import DirectView, LoadBalancedView
 
 #--------------------------------------------------------------------------
@@ -101,17 +100,19 @@ class ExecuteReply(RichOutput):
         execute_result = self.metadata['execute_result']
         if execute_result:
             return execute_result.get('data', {})
+        return {}
     
     @property
     def _metadata(self):
         execute_result = self.metadata['execute_result']
         if execute_result:
             return execute_result.get('metadata', {})
-    
+        return {}
+
     def display(self):
         from IPython.display import publish_display_data
         publish_display_data(self.data, self.metadata)
-    
+
     def _repr_mime_(self, mime):
         if mime not in self.data:
             return
@@ -120,52 +121,64 @@ class ExecuteReply(RichOutput):
             return data, self._metadata[mime]
         else:
             return data
-    
+
+    def _repr_mimebundle_(self, *args, **kwargs):
+        data, md = self.data, self.metadata
+        if 'text/plain' in data:
+            data = data.copy()
+            data['text/plain'] = self._plaintext()
+        return data, md
+
     def __getitem__(self, key):
         return self.metadata[key]
-    
+
     def __getattr__(self, key):
         if key not in self.metadata:
             raise AttributeError(key)
         return self.metadata[key]
-    
+
     def __repr__(self):
         execute_result = self.metadata['execute_result'] or {'data':{}}
         text_out = execute_result['data'].get('text/plain', '')
         if len(text_out) > 32:
             text_out = text_out[:29] + '...'
-        
+
         return "<ExecuteReply[%i]: %s>" % (self.execution_count, text_out)
-    
-    def _repr_pretty_(self, p, cycle):
+
+    def _plaintext(self):
         execute_result = self.metadata['execute_result'] or {'data':{}}
         text_out = execute_result['data'].get('text/plain', '')
-        
+
         if not text_out:
-            return
-        
-        try:
-            ip = get_ipython()
-        except NameError:
+            return ''
+
+        ip = get_ipython()
+        if ip is None:
             colors = "NoColor"
         else:
             colors = ip.colors
-        
+
         if colors == "NoColor":
             out = normal = ""
         else:
             out = TermColors.Red
             normal = TermColors.Normal
-        
+
         if '\n' in text_out and not text_out.startswith('\n'):
             # add newline for multiline reprs
             text_out = '\n' + text_out
-        
-        p.text(
-            out + u'Out[%i:%i]: ' % (
+
+        return u''.join([
+            out,
+            u'Out[%i:%i]: ' % (
                 self.metadata['engine_id'], self.execution_count
-            ) + normal + text_out
-        )
+            ),
+            normal,
+            text_out,
+            ])
+
+    def _repr_pretty_(self, p, cycle):
+        p.text(self._plaintext())
 
 
 class Metadata(dict):
@@ -385,8 +398,10 @@ class Client(HasTraits):
                 short = compress_user(url_file)
                 if not os.path.exists(url_file):
                     print("Waiting for connection file: %s" % short)
-                    for i in range(30):
-                        time.sleep(1)
+                    waiting_time = 0.
+                    while waiting_time < timeout:
+                        time.sleep(min(timeout-waiting_time, 1))
+                        waiting_time += 1
                         if os.path.exists(url_file):
                             break
                 if not os.path.exists(url_file):
@@ -427,18 +442,19 @@ class Client(HasTraits):
         
         if location is not None and addr == localhost():
             # location specified, and connection is expected to be local
-            if not is_local_ip(location) and not sshserver:
+            location_ip = util.ip_for_host(location)
+            if not is_local_ip(location_ip) and not sshserver:
                 # load ssh from JSON *only* if the controller is not on
                 # this machine
                 sshserver=cfg['ssh']
-            if not is_local_ip(location) and not sshserver:
+            if not is_local_ip(location_ip) and not sshserver:
                 # warn if no ssh specified, but SSH is probably needed
                 # This is only a warning, because the most likely cause
                 # is a local Controller on a laptop whose IP is dynamic
                 warnings.warn("""
             Controller appears to be listening on localhost, but not on this machine.
             If this is true, you should specify Client(...,sshserver='you@%s')
-            or instruct your controller to listen on an external IP."""%location,
+            or instruct your controller to listen on an external IP.""" % location,
                 RuntimeWarning)
         elif not sshserver:
             # otherwise sync with cfg
@@ -477,7 +493,8 @@ class Client(HasTraits):
 
         if self._ssh:
             from zmq.ssh import tunnel
-            tunnel.tunnel_connection(self._query_socket, cfg['registration'], sshserver, **ssh_kwargs)
+            tunnel.tunnel_connection(self._query_socket, cfg['registration'], sshserver,
+                                     timeout=timeout, **ssh_kwargs)
         else:
             self._query_socket.connect(cfg['registration'])
 
@@ -498,9 +515,8 @@ class Client(HasTraits):
         
         # last step: setup magics, if we are in IPython:
         
-        try:
-            ip = get_ipython()
-        except NameError:
+        ip = get_ipython()
+        if ip is None:
             return
         else:
             if 'px' not in ip.magics_manager.magics:
@@ -664,7 +680,7 @@ class Client(HasTraits):
         msg_meta = msg['metadata']
         content = msg['content']
         md = {'msg_id' : parent['msg_id'],
-              'received' : datetime.now(),
+              'received' : util.utcnow(),
               'engine_uuid' : msg_meta.get('engine', None),
               'follow' : msg_meta.get('follow', []),
               'after' : msg_meta.get('after', []),
@@ -677,7 +693,7 @@ class Client(HasTraits):
         if 'date' in parent:
             md['submitted'] = parent['date']
         if 'started' in msg_meta:
-            md['started'] = parse_date(msg_meta['started'])
+            md['started'] = util._parse_date(msg_meta['started'])
         if 'date' in header:
             md['completed'] = header['date']
         return md
@@ -824,7 +840,7 @@ class Client(HasTraits):
 
     def _make_io_loop(self):
         """Make my IOLoop. Override with IOLoop.current to return"""
-        return IOLoop()
+        return IOLoop(make_current=False)
     
     def _stop_io_thread(self):
         """Stop my IO thread"""
@@ -855,6 +871,7 @@ class Client(HasTraits):
     
     def _io_main(self):
         """main loop for background IO thread"""
+        self._io_loop.make_current()
         self._io_loop.start()
         self._io_loop.close()
     
@@ -956,7 +973,7 @@ class Client(HasTraits):
         if asyncresult:
             future.output = output
             futures.append(output)
-            output.metadata['submitted'] = datetime.now()
+            output.metadata['submitted'] = util.utcnow()
         
         def cleanup(f):
             """Purge caches on Future resolution"""
@@ -1169,7 +1186,7 @@ class Client(HasTraits):
         targets = self._build_targets(targets)[0]
         futures = []
         for t in targets:
-            futures.append(self._send(self._control_socket, 'clear_request', content={}, ident=t))
+            futures.append(self._send(self._control_stream, 'clear_request', content={}, ident=t))
         if not block:
             return multi_future(futures)
         for future in futures:
@@ -1212,7 +1229,7 @@ class Client(HasTraits):
         content = dict(msg_ids=msg_ids)
         futures = []
         for t in targets:
-            futures.append(self._send(self._control_socket, 'abort_request',
+            futures.append(self._send(self._control_stream, 'abort_request',
                     content=content, ident=t))
         
         if not block:
@@ -1254,7 +1271,7 @@ class Client(HasTraits):
         
         futures = []
         for t in targets:
-            futures.append(self._send(self._control_socket, 'shutdown_request',
+            futures.append(self._send(self._control_stream, 'shutdown_request',
                         content={'restart':restart},ident=t))
         error = False
         if block or hub:
@@ -1267,7 +1284,7 @@ class Client(HasTraits):
         if hub:
             # don't trigger close on shutdown notification, which will prevent us from receiving the reply
             self._notification_handlers['shutdown_notification'] = lambda msg: None
-            msg = self._send_recv(self._query_socket, 'shutdown_request')
+            msg = self._send_recv(self._query_stream, 'shutdown_request')
             if msg['content']['status'] != 'ok':
                 error = self._unwrap_exception(msg['content'])
             if not error:
@@ -1276,14 +1293,14 @@ class Client(HasTraits):
         if error:
             raise error
 
-    def become_distributed(self, targets='all', port=0, nanny=False, scheduler_args=None, **worker_args):
+    def become_dask(self, targets='all', port=0, nanny=False, scheduler_args=None, **worker_args):
         """Turn the IPython cluster into a dask.distributed cluster
 
         Parameters
         ----------
 
         targets: target spec (default: all)
-            Which engines to turn into distributed workers.
+            Which engines to turn into dask workers.
         port: int (default: random)
             Which port
         nanny: bool (default: False)
@@ -1297,8 +1314,8 @@ class Client(HasTraits):
         Returns
         -------
 
-        executor: distributed.Executor
-            A distributed.Executor connected to the distributed cluster.
+        client = distributed.Client
+            A dask.distributed.Client connected to the dask cluster.
         """
         import distributed
 
@@ -1310,7 +1327,7 @@ class Client(HasTraits):
             scheduler_args = dict(scheduler_args) # copy
 
         # Start a Scheduler on the Hub:
-        reply = self._send_recv(self._query_socket, 'become_distributed_request',
+        reply = self._send_recv(self._query_stream, 'become_dask_request',
             {'scheduler_args': scheduler_args},
         )
         if reply['content']['status'] != 'ok':
@@ -1323,31 +1340,43 @@ class Client(HasTraits):
         worker_args['nanny'] = nanny
         # set default ncores=1, since that's how an IPython cluster is typically set up.
         worker_args.setdefault('ncores', 1)
-        dview.apply_sync(util.become_distributed_worker, **worker_args)
+        dview.apply_sync(util.become_dask_worker, **worker_args)
 
-        # Finally, return an Executor connected to the Scheduler
-        executor = distributed.Executor('{ip}:{port}'.format(**distributed_info))
-        return executor
+        # Finally, return a Client connected to the Scheduler
+        try:
+            distributed_Client = distributed.Client
+        except AttributeError:
+            # For distributed pre-1.18.1
+            distributed_Client = distributed.Executor
 
-    def stop_distributed(self, targets='all'):
-        """Stop the distributed Scheduler and Workers started by become_distributed.
+        client = distributed_Client('{ip}:{port}'.format(**distributed_info))
+
+        return client
+
+
+    def stop_dask(self, targets='all'):
+        """Stop the distributed Scheduler and Workers started by become_dask.
 
         Parameters
         ----------
 
         targets: target spec (default: all)
-            Which engines to turn into distributed workers.
+            Which engines to stop dask workers on.
         """
         dview = self.direct_view(targets)
 
         # Start a Scheduler on the Hub:
-        reply = self._send_recv(self._query_socket, 'stop_distributed_request')
+        reply = self._send_recv(self._query_stream, 'stop_distributed_request')
         if reply['content']['status'] != 'ok':
             raise self._unwrap_exception(reply['content'])
 
         # Finally, stop all the Workers on the engines
         dview.apply_sync(util.stop_distributed_worker)
-
+    
+    # aliases:
+    become_distributed = become_dask
+    stop_distributed = stop_dask
+    
     #--------------------------------------------------------------------------
     # Execution related methods
     #--------------------------------------------------------------------------
@@ -1375,7 +1404,7 @@ class Client(HasTraits):
         metadata = metadata if metadata is not None else {}
 
         # validate arguments
-        if not callable(f) and not isinstance(f, Reference):
+        if not callable(f) and not isinstance(f, (Reference, PrePickled)):
             raise TypeError("f must be callable, not %s"%type(f))
         if not isinstance(args, (tuple, list)):
             raise TypeError("args must be tuple or list, not %s"%type(args))
@@ -1438,7 +1467,7 @@ class Client(HasTraits):
                 # save for later, in case of engine death
                 self._outstanding_dict[ident].add(msg_id)
         self.history.append(msg_id)
-        self.metadata[msg_id]['submitted'] = datetime.now()
+        self.metadata[msg_id]['submitted'] = util.utcnow()
 
         return future
 
@@ -1463,7 +1492,7 @@ class Client(HasTraits):
             targets = None
         if targets is not None:
             targets = self._build_targets(targets)[1]
-        return LoadBalancedView(client=self, socket=self._task_socket, targets=targets,
+        return LoadBalancedView(client=self, socket=self._task_stream, targets=targets,
                                 **kwargs)
     
     def executor(self, targets=None):
@@ -1508,7 +1537,7 @@ class Client(HasTraits):
             targets = self._build_targets(targets)[1]
         if single:
             targets = targets[0]
-        return DirectView(client=self, socket=self._mux_socket, targets=targets,
+        return DirectView(client=self, socket=self._mux_stream, targets=targets,
                           **kwargs)
 
     #--------------------------------------------------------------------------
@@ -1597,7 +1626,7 @@ class Client(HasTraits):
         theids = self._msg_ids_from_jobs(indices_or_msg_ids)
         content = dict(msg_ids = theids)
 
-        reply = self._send_recv(self._query_socket, 'resubmit_request', content)
+        reply = self._send_recv(self._query_stream, 'resubmit_request', content)
         content = reply['content']
         if content['status'] != 'ok':
             raise self._unwrap_exception(content)
@@ -1649,7 +1678,7 @@ class Client(HasTraits):
 
         if theids: # some not locally cached
             content = dict(msg_ids=theids, status_only=status_only)
-            reply = self._send_recv(self._query_socket, "result_request", content=content)
+            reply = self._send_recv(self._query_stream, "result_request", content=content)
             content = reply['content']
             if content['status'] != 'ok':
                 raise self._unwrap_exception(content)
@@ -1670,8 +1699,8 @@ class Client(HasTraits):
         for msg_id in sorted(theids):
             if msg_id in content['completed']:
                 rec = content[msg_id]
-                parent = extract_dates(rec['header'])
-                header = extract_dates(rec['result_header'])
+                parent = util.extract_dates(rec['header'])
+                header = util.extract_dates(rec['result_header'])
                 rcontent = rec['result_content']
                 iodict = rec['io']
                 if isinstance(rcontent, str):
@@ -1686,7 +1715,7 @@ class Client(HasTraits):
                 )
                 md.update(self._extract_metadata(md_msg))
                 if rec.get('received'):
-                    md['received'] = parse_date(rec['received'])
+                    md['received'] = util._parse_date(rec['received'])
                 md.update(iodict)
                 
                 if rcontent['status'] == 'ok':
@@ -1727,7 +1756,7 @@ class Client(HasTraits):
         else:
             engine_ids = self._build_targets(targets)[1]
         content = dict(targets=engine_ids, verbose=verbose)
-        reply = self._send_recv(self._query_socket, "queue_request", content=content)
+        reply = self._send_recv(self._query_stream, "queue_request", content=content)
         content = reply['content']
         status = content.pop('status')
         if status != 'ok':
@@ -1892,7 +1921,7 @@ class Client(HasTraits):
             msg_ids = self._msg_ids_from_jobs(jobs)
 
         content = dict(engine_ids=targets, msg_ids=msg_ids)
-        reply = self._send_recv(self._query_socket, "purge_request", content=content)
+        reply = self._send_recv(self._query_stream, "purge_request", content=content)
         content = reply['content']
         if content['status'] != 'ok':
             raise self._unwrap_exception(content)
@@ -1948,7 +1977,7 @@ class Client(HasTraits):
                 list of all msg_ids, ordered by task submission time.
         """
 
-        reply = self._send_recv(self._query_socket, "history_request", content={})
+        reply = self._send_recv(self._query_stream, "history_request", content={})
         content = reply['content']
         if content['status'] != 'ok':
             raise self._unwrap_exception(content)
@@ -1972,7 +2001,7 @@ class Client(HasTraits):
         if isinstance(keys, string_types):
             keys = [keys]
         content = dict(query=query, keys=keys)
-        reply = self._send_recv(self._query_socket, "db_request", content=content)
+        reply = self._send_recv(self._query_stream, "db_request", content=content)
         content = reply['content']
         if content['status'] != 'ok':
             raise self._unwrap_exception(content)
@@ -1988,10 +2017,10 @@ class Client(HasTraits):
             # unpack datetime objects
             for hkey in ('header', 'result_header'):
                 if hkey in rec:
-                    rec[hkey] = extract_dates(rec[hkey])
+                    rec[hkey] = util.extract_dates(rec[hkey])
             for dtkey in ('submitted', 'started', 'completed', 'received'):
                 if dtkey in rec:
-                    rec[dtkey] = parse_date(rec[dtkey])
+                    rec[dtkey] = util._parse_date(rec[dtkey])
             # relink buffers
             if has_bufs:
                 blen = buffer_lens[i]
